@@ -46,12 +46,16 @@
 #endif
 
 #include <libxfce4util/i18n.h>
+#include <libxfce4util/debug.h>
 #include <libxfcegui4/session-client.h>
 #include <libxfcegui4/netk-util.h>
 
 #include "xfce.h"
 #include "settings.h"
 #include "mcs_client.h"
+#include "controls_dialog.h"
+#include "item_dialog.h"
+#include "popup.h"
 
 /* signal handling */
 typedef enum
@@ -59,12 +63,15 @@ typedef enum
     NOSIGNAL,
     RESTART,
     QUIT,
+    QUIT_CONFIRM,
     NUM_SIGNALS,
 }
 SignalState;
 
 static SignalState sigstate = NOSIGNAL;
 
+static char *progname = NULL;
+
 /*  session management
  *  ------------------
 */
@@ -74,29 +81,8 @@ static gboolean session_managed = FALSE;
 static void
 save_panel (void)
 {
-#if 0
-    gboolean hidden = settings.autohide;
-
-    if (panel.toplevel)
-    {
-	if (hidden)
-	{
-	    panel_set_autohide (FALSE);
-
-	    while (gtk_events_pending ())
-		gtk_main_iteration ();
-	}
-
-	gtk_window_get_position (GTK_WINDOW (panel.toplevel),
-				 &panel.position.x, &panel.position.y);
-
-	gtk_widget_hide (panel.toplevel);
-    }
-
-    if (hidden && panel.toplevel)
-	panel_set_autohide (TRUE);
-#endif
-    write_panel_config ();
+    if (!disable_user_config)
+	write_panel_config ();
 }
 
 static void
@@ -145,34 +131,12 @@ quit (gboolean force)
     if (gtk_main_level ())
 	gtk_main_quit ();
 
-    exit (0);
-}
-
-void
-restart (void)
-{
-/*    int x, y;
-    
-    x = panel.position.x;
-    y = panel.position.y;
-*/
-    save_panel ();
-
-    if (panel.toplevel)
-	gtk_widget_hide (panel.toplevel);
-
-#ifdef HAVE_LIBSTARTUP_NOTIFICATION
-    free_startup_timeout ();
-#endif
-
-    panel_cleanup ();
-    gtk_widget_destroy (panel.main_frame);
-
-    create_panel ();
-/*
-    panel.position.x = x;
-    panel.position.y = y;
-    panel_set_position ();*/
+    DBG("sigstate: %d", sigstate);
+    if (sigstate != RESTART)
+    {
+	g_message("%s: Exit", PACKAGE);
+	exit (0);
+    }
 }
 
 /*  Main program
@@ -185,17 +149,31 @@ check_signal_state (void)
 
     if (sigstate != NOSIGNAL)
     {
+	/* close open dialogs */
+	destroy_controls_dialog();
+	destroy_menu_dialog();
+	
 	if (sigstate == RESTART && !restarting)
 	{
 	    restarting = TRUE;
-	    sigstate = NOSIGNAL;
-	    restart ();
-	    restarting = FALSE;
+	    
+	    /* calls gtk_main_quit() */
+	    quit(TRUE);
+
+	    /* progname is saved on startup 
+	     * TODO: do we need to pass on arguments? */
+	    g_message("%s: restarting %s ...", PACKAGE, progname);
+	    execlp(progname,progname,NULL);
 	}
 	else if (sigstate == QUIT)
 	{
 	    quit (TRUE);
 	}
+	else if (sigstate == QUIT_CONFIRM)
+	{
+	    sigstate = NOSIGNAL;
+	    quit (FALSE);
+	}
     }
 
     /* keep running */
@@ -216,6 +194,10 @@ sighandler (int sig)
 	    sigstate = RESTART;
 	    break;
 
+	case SIGUSR2:
+	    sigstate = QUIT_CONFIRM;
+	    break;
+	    
 	case SIGINT:
 	    /* hack: prevent the panel from saving config on ^C */
 	    disable_user_config = TRUE;
@@ -235,6 +217,8 @@ main (int argc, char **argv)
     gboolean net_wm_support;
     int i;
 
+    progname = argv[0];
+    
     net_wm_support = FALSE;
 
 #if 0
@@ -277,6 +261,31 @@ main (int argc, char **argv)
 		    "http://www.freedesktop.org. \n"
 		    "Some XFce features may not work as intended."));
     }
+#ifdef HAVE_SIGACTION
+    act.sa_handler = sighandler;
+    sigemptyset (&act.sa_mask);
+#ifdef SA_RESTART
+    act.sa_flags = SA_RESTART;
+#else
+    act.sa_flags = 0;
+#endif
+    sigaction (SIGHUP, &act, NULL);
+    sigaction (SIGUSR1, &act, NULL);
+    sigaction (SIGUSR2, &act, NULL);
+    sigaction (SIGINT, &act, NULL);
+    sigaction (SIGTERM, &act, NULL);
+#else
+    signal (SIGHUP, sighandler);
+    signal (SIGUSR1, sighandler);
+    signal (SIGUSR2, sighandler);
+    signal (SIGINT, sighandler);
+    signal (SIGTERM, sighandler);
+#endif
+
+    /* icon framework: names and id's */
+    icons_init ();
+
+    create_panel ();
 
     client_session = client_session_new (argc, argv, NULL /* data */ ,
 					 SESSION_RESTART_IF_RUNNING, 40);
@@ -287,28 +296,6 @@ main (int argc, char **argv)
     if (!(session_managed = session_init (client_session)))
 	g_message ("%s: Running without session manager", PACKAGE);
 
-#ifdef HAVE_SIGACTION
-    act.sa_handler = sighandler;
-    sigemptyset (&act.sa_mask);
-#ifdef SA_RESTART
-    act.sa_flags = SA_RESTART;
-#else
-    act.sa_flags = 0;
-#endif
-    sigaction (SIGUSR1, &act, NULL);
-    sigaction (SIGINT, &act, NULL);
-    sigaction (SIGTERM, &act, NULL);
-#else
-    signal (SIGUSR1, sighandler);
-    signal (SIGINT, sighandler);
-    signal (SIGTERM, sighandler);
-#endif
-
-    /* icon framework: names and id's */
-    icons_init ();
-
-    create_panel ();
-
     /* signal state */
     g_timeout_add (500, (GSourceFunc) check_signal_state, NULL);
 
